package com.bubbleoverlay

import android.app.Activity
import android.content.ComponentName
import android.content.Context
import android.content.Intent
import android.content.ServiceConnection
import android.net.Uri
import android.os.Build
import android.os.IBinder
import android.provider.Settings
import androidx.annotation.NonNull
import io.flutter.embedding.android.FlutterActivity
import io.flutter.embedding.engine.FlutterEngine
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel


class BubbleOverlayPlugin : FlutterActivity(), FlutterPlugin, MethodChannel.MethodCallHandler {
    private lateinit var channel: MethodChannel
    private val channelName: String = "bubble_overlay"
    private val CODE_DRAW_OVER_OTHER_APP_PERMISSION = 2084
    private lateinit var mOverlayService: BubbleOverlayService
    private var mBound: Boolean = false
    private lateinit var connection: ServiceConnection

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        when (call.method) {
            "openBubble" -> {
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M && !Settings.canDrawOverlays(this)) {
                    //If the draw over permission is not available open the settings screen
                    //to grant the permission.
                    startActivityForResult(Intent(Settings.ACTION_MANAGE_OVERLAY_PERMISSION, Uri.parse("package:$packageName")), CODE_DRAW_OVER_OTHER_APP_PERMISSION)
                } else if (!mBound) startService(call.arguments as List<String>)
            }
            "updateBubbleText" -> {
                if (mBound) {
                    val text = call.arguments as String
                    mOverlayService.updateText(text)
                } else
                    throw Exception("BubbleService not running.")
            }
            "updateBubbleTextColor" -> {
                if (mBound) {
                    val textColor = call.arguments as String
                    mOverlayService.updateTextColor(textColor)
                } else
                    throw Exception("BubbleService not running.")
            }
            "updateBubbleColor" -> {
                if (mBound) {
                    val bubbleColor = call.arguments as String
                    mOverlayService.updateBubbleColor(bubbleColor)
                } else
                    throw Exception("BubbleService not running.")
            }
            "updateBubbleTopIcon" -> {
                if (mBound) {
                    val icon = call.arguments as ByteArray
                    mOverlayService.updateIconTop(icon)
                } else
                    throw Exception("BubbleService not running.")
            }
            "updateBubbleBottomIcon" -> {
                if (mBound) {
                    val icon = call.arguments as ByteArray
                    mOverlayService.updateIconBottom(icon)
                } else
                    throw Exception("BubbleService not running.")
            }
            else -> result.notImplemented()
        }
    }

    override fun configureFlutterEngine(@NonNull flutterEngine: FlutterEngine) {
        MethodChannel(flutterEngine.dartExecutor.binaryMessenger, channelName).setMethodCallHandler(this)
    }

    override fun onDetachedFromEngine(binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.getFlutterEngine().dartExecutor, channelName)
        channel.setMethodCallHandler(this);
    }


    private fun startService(arguments: List<Any?>) {
        startService(Intent(this, BubbleOverlayService::class.java))
        connection = object : ServiceConnection {

            override fun onServiceConnected(className: ComponentName, service: IBinder) {
                val binder = service as BubbleOverlayService.LocalBinder
                mOverlayService = binder.getService()
                mBound = true

                val customText: String? = arguments[0] as String?
                val textColor: String? = arguments[1] as String?
                val backgroundColor: String? = arguments[2] as String?
                val topIconAsset: ByteArray? = arguments[3] as ByteArray?
                val bottomIconAsset: ByteArray? = arguments[4] as ByteArray?

                if (customText != null)
                    mOverlayService.updateText(customText)
                if (textColor != null)
                    mOverlayService.updateTextColor(textColor)
                if (backgroundColor != null)
                    mOverlayService.updateBubbleColor(backgroundColor)
                if (topIconAsset != null)
                    mOverlayService.updateIconTop(topIconAsset)
                if (bottomIconAsset != null)
                    mOverlayService.updateIconBottom(bottomIconAsset)

            }

            override fun onServiceDisconnected(arg0: ComponentName) {
                mBound = false
            }
        }

        Intent(this, BubbleOverlayService::class.java).also { i -> bindService(i, connection, Context.BIND_AUTO_CREATE) }


        val startMain = Intent(Intent.ACTION_MAIN)
        startMain.addCategory(Intent.CATEGORY_HOME)
        startMain.flags = Intent.FLAG_ACTIVITY_NEW_TASK
        startActivity(startMain)
    }


    override fun onStop() {
        super.onStop()
        unbindService(connection)
        mBound = false
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        if (requestCode == CODE_DRAW_OVER_OTHER_APP_PERMISSION) if (resultCode != Activity.RESULT_OK)
            throw Exception("Draw over other app permission not available. Closing the application")
        super.onActivityResult(requestCode, resultCode, data)
    }

}